/* $Id: CheckBoxP.h,v 1.1 1997/04/15 10:07:05 archer Exp $
 *	CHECKBOXP.H - Private structures needed by Checkbox / Radiobox
 */
#ifndef _widget_checkboxp_h_included
#define _widget_checkboxp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/LabelP.h>
#include <Widget/CheckBox.h>


/**
 ** Private types
 **/
typedef struct {
/*
 * Checkbox is subclassed from Label
 */
	x11__label	label;
/*
 * Callback
 */
	X11_callback	on_toggle;
	void		*data;
/*
 * Private parts
 */
	int		box_width;
	boolean		radio_look;
	boolean		checked;
} x11__checkbox;
#define W_CHECKBOX		((x11__checkbox *)wdg)


#endif /*_widget_checkboxp_h_included*/
