/* $Id: Alarm.h,v 1.1 1997/04/15 10:04:03 archer Exp $
 *	ALARM.H - Modal PoPup alarm window
 *
 * Include generic "continue" windows, choice windows, and system error windows
 */
#ifndef _widget_alarm_h_included
#define _widget_alarm_h_included
#include <Widget/Widgets.h>
#include <Widget/Icon.h>


/**
 ** Prototypes of alarms
 **/
extern int	X11_AlarmPopup(char *,X11_icon *,char *,char *,char *);
extern void	X11_AlarmSyserr(char *,char *);


#endif /*_widget_alarm_h_included*/
