/* $Id: PerrorP.h,v 1.1 1997/04/15 10:07:13 archer Exp $
 *	PERRORP.H - Handle X11 fatal errors
 *
 * All the error resources are pre-loaded, to be available "in case".
 */
#ifndef _widget_errorsp_h_included
#define _widget_errorsp_h_included
#include <Widget/Perror.h>


/**
 ** Internal resources
 **/
extern Window	x11__error_window;


/**
 ** Semi-public prototypes
 **/
extern void	x11__error_setup(void);


#endif /*_widget_errorsp_h_included*/
