/* $Id: Arrow.h,v 1.1 1997/04/17 13:20:26 archer Exp $
 *	ARROW.H - Arrow management
 *
 * How to draw an arrow button, if needed
 */
#ifndef _widget_arrow_h_included
#define _widget_arrow_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Arrow(X11_widget,int,X11_callback,void *);
#define X11_ARROW_DOWN		0
#define X11_ARROW_LEFT		1
#define X11_ARROW_UP		2
#define X11_ARROW_RIGHT		3


#endif /*_widget_arrow_h_included*/
