/* $Id: MenuButton.h,v 1.2 1997/04/15 13:21:10 archer Exp $
 *	MENUBUTTON.H - Widget Menu Button
 *
 * This is a handle to selection menus
 */
#ifndef _widget_menubutton_h_included
#define _widget_menubutton_h_included
#include <Widget/Widgets.h>
#include <Widget/Label.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_MenuButton(X11_widget,char *,char *);
#define X11_MenuButtonSetString(w,t) X11_LabelSetString(w,t)


#endif /*_widget_menubutton_h_included*/
