/* $Id: Form.h,v 1.3 1997/05/10 09:21:17 archer Exp $
 *	FORM.H - Forms type container
 *
 * Elementary managers
 */
#ifndef _widget_form_h_included
#define _widget_form_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Column(X11_widget,char *);
extern X11_widget	X11_Row(X11_widget,char *);
extern X11_widget	X11_RowColumn(X11_widget,char *);

extern void		X11_FormMargins(X11_widget,unsigned int,unsigned int);
extern void		X11_FormAbsoluteX(X11_widget,int);
extern void		X11_FormAlignLabels(X11_widget *,int);
extern void		X11_FormAlignRight(X11_widget *,int);
extern void		X11_FormFixedWidth(X11_widget);
extern void		X11_FormLabel(X11_widget);


#endif /*_widget_form_h_included*/
