/* $Id: Menu.h,v 1.3 1997/06/27 12:40:57 archer Exp $
 *	MENU.H - Widget Menu
 *
 * Menus are containers for MenuItems (plus a label, aka Menu title)
 */
#ifndef _widget_menu_h_included
#define _widget_menu_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Menu(X11_widget,char *);
extern X11_widget	X11_MenuByIndex(X11_widget,int);
extern void		X11_MenuEmpty(X11_widget);
extern int		X11_MenuIndex(X11_widget,X11_widget);
extern void		X11_MenuOnPopup(X11_widget,X11_callback,void *);
extern void		X11_MenuOpenCall(X11_widget,X11_callback,void *);
extern void		X11_MenuPop(X11_widget,int,int);
extern void		X11_MenuTitle(X11_widget,char *);
extern void		*X11_MenuUserData(X11_widget);
extern void		X11_MenuTruncate(X11_widget,X11_widget);


#endif /*_widget_menu_h_included*/
