/* $Id: Perror.c,v 1.1 1997/04/15 10:09:44 archer Exp $
 *	PERROR.C - Display current error. This is fatal (always!)
 *
 * The purpose is to display a window (as aesthetically looking as possible)
 * while using as few resources as possible, since we're probably in a failure
 * mode.
 */
#include <Widget/WidgetsP.h>
#include <Widget/PerrorP.h>
#include <stdio.h>


/**
 ** Semi-global variable
 **/
Window	x11__error_window;


/**
 ** X11_ERROR_SETUP
 **	Pre-allocate the resources needed by the Error setup
 **/
void x11__error_setup(void)
{
	x11__error_window = XCreateSimpleWindow(x11__screen, x11__root_id,
					0, 0, 400, 200, 0, 0, x11__white_pixel);
/*** HACK ***/
}


/**
 ** X11_PERROR
 **	Print error
 **/
void X11_Perror(char *txt)
{
/*** HACK ***/
	perror(txt);
	exit(1);
}
