/* $Id: EventsP.h,v 1.1 1997/04/15 10:07:07 archer Exp $
 *	EVENTSP.H - Private declarations for event-handling functions
 *
 * Include the defer type, and the most common functions
 */
#ifndef _widget_eventsp_h_included
#define _widget_eventsp_h_included
#include <Widget/Events.h>
#include <Widget/WidgetsP.h>


/**
 ** Internal variables
 **/
extern X11_widget	x11__widget_garbage_bin;
extern fd_set		x11__widget_ifds;
extern fd_set		x11__widget_ofds;


/**
 ** Prototypes for events
 **/
extern void	x11__event_dispatch(XEvent *);
extern void	x11__wait_event(XEvent *);


#endif /*_widget_eventsp_h_included*/
