/* $Id: ScrollerP.h,v 1.1 1997/04/17 13:20:45 archer Exp $
 *	SCROLLERP.H - Private structures needed by Scroller
 *
 * Scroller is never created directly; it's always used by another object as
 * its manager
 */
#ifndef _widget_scrollerp_h_included
#define _widget_scrollerp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/Scroller.h>
#include <Widget/ScrollbarP.h>


/**
 ** Private types
 **/
typedef struct {
	x11__core	core;
/*
 * Private parts
 */
	X11_widget	horizontal_bar;
	X11_widget	vertical_bar;
	X11_widget	window;
} x11__scroller;
#define W_SCROLLER	((x11__scroller *)wdg)


#endif /*_widget_scrollerp_h_included*/
