/* $Id: fontsP.h,v 1.1 1997/04/15 10:07:20 archer Exp $
 *	FONTSP.H - Font caching and parsing
 *
 * Include font searching, and common (read "used by multiple widgets") names.
 */
#ifndef _widget_fontsp_h_included
#define _widget_fontsp_h_included
#include <Widget/WidgetsP.h>


/**
 ** Private types
 **/
typedef struct x11__font {
	struct x11__font	*next;
	XFontStruct		*info;
	char			*name;
} x11__font;


/**
 ** Common font names
 **/
extern char	x11__font_courrier12[];
extern char	x11__font_helvetica10[];
extern char	x11__font_helvetica12[];
extern char	x11__font_helveticabold12[];
extern char	x11__font_lucida12[];
extern char	x11__font_lucidabold12[];


/**
 ** Prototypes
 **/
extern XFontStruct	*x11__font_find(char *);
extern XFontStruct	*x11__font_resource(char *);


#endif /*_widget_fontsp_h_included*/
