/* $Id: MenuP.h,v 1.1 1997/04/15 10:07:12 archer Exp $
 *	MENUP.H - Private structures needed by Menu popups
 *
 * A Menu may be either generic or have popup
 */
#ifndef _widget_menup_h_included
#define _widget_menup_h_included
#include <Widget/WidgetsP.h>
#include <Widget/LabelP.h>
#include <Widget/Menu.h>


/**
 ** Private types
 **/
typedef struct {
	x11__label	label;
/*
 * Private part
 */
	X11_callback	on_popup;
	void		*user_data;
} x11__menu;
#define W_MENU		((x11__menu *)wdg)


/**
 ** Private variables
 **/
extern char	x11__menu_class[];


/**
 ** Prototypes
 **/
extern void	x11__menu_open(X11_widget,XEvent *);
extern void	x11__menu_pop(X11_widget);


#endif /*_widget_menup_h_included*/
