/* $Id: MenuButtonP.h,v 1.2 1997/04/15 13:21:26 archer Exp $
 *	MENUBUTTONP.H - Structure used for Menu buttons
 *
 * MenuButtons are sub-classes of basic Buttons.
 */
#ifndef _widget_menubuttonp_h_included
#define _widget_menubuttonp_h_included
#include <Widget/MenuButton.h>
#include <Widget/ButtonP.h>


/**
 ** Private types
 **/
typedef struct {
	x11__button	button;
/*
 * Private parts
 */
	int		side_margin;
	int		left_margin, right_margin;
} x11__menubutton;
#define W_MENUBUTTON	((x11__menubutton *)wdg)
#define C_MENUBUTTON	((x11__menubutton *)child)


/**
 ** Prototypes
 **/
extern void	x11__menub_geometry(x11__menubutton *);
extern void	x11__menub_resources(x11__menubutton *,char *,char *);
extern void	x11__menub_init(x11__menubutton *,char *);


#endif /*_widget_menubuttonp_h_included*/
