/* $Id: drawP.h,v 1.3 1997/06/10 15:29:24 archer Exp $
 *	DRAWP.H - Prototypes for common drawing functions
 *
 * These functions are used by everyone
 */
#ifndef _widget_drawp_h_included
#define _widget_drawp_h_included
#include <Widget/colorsP.h>


/**
 ** Global variables
 **/
extern GC		x11__default_gc;
extern GC		x11__stippled_gc,
			x11__fore_gc,
			x11__back_gc,
			x11__active_gc,
			x11__topleft_gc,
			x11__bottomright_gc;
extern Window		x11__cur_id;
extern Pixmap		x11__stipple;
extern Pixmap		x11__stipple_light;
extern X11_widget	x11__draw_in;
extern int		x11__x_left,
			x11__y_top,
			x11__x_right,
			x11__y_bot;
extern int		x11__x_w,
			x11__y_h;


/**
 ** Prototypes
 **/
extern void	x11__draw_downshadow(unsigned int);
extern void	x11__draw_fit_h(int);
extern void	x11__draw_fit_v(int);
extern void	x11__draw_init(void);
extern void	x11__draw_lined(X11_color,unsigned int);
extern void	x11__draw_filled(X11_color);
extern void	x11__draw_nothing(unsigned int);
extern void	x11__draw_push(void);
extern void	x11__draw_pop(void);
extern void	x11__draw_start(X11_widget);
extern void	x11__draw_upshadow(unsigned int);
extern void	x11__shadows_gc(void);


#endif /*_widget_drawp_h_included*/
