/* $Id: Label.h,v 1.1 1997/04/15 10:04:09 archer Exp $
 *	LABEL.H - Widget Label
 *
 * Usable for both fixed-type labels and variable-contents labels
 */
#ifndef _widget_label_h_included
#define _widget_label_h_included
#include <Widget/Widgets.h>
#include <Widget/Icon.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Label(X11_widget,char *,char *);
extern char		*X11_LabelGetString(X11_widget);
extern void		X11_LabelSetIcon(X11_widget,X11_icon *);
extern void		X11_LabelSetString(X11_widget,char *);


#endif /*_widget_label_h_included*/
