/* $Id: MenuItem.h,v 1.4 1997/06/27 12:40:58 archer Exp $
 *	MENUITEM.H - Widget Menu Item
 *
 * MenuItems appear within menus, and allow other menus to pop, or actions
 */
#ifndef _widget_menuitem_h_included
#define _widget_menuitem_h_included
#include <Widget/Widgets.h>
#include <Widget/Label.h>
#include <Widget/Button.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_MenuItem(X11_widget,char *,char *,char,
						X11_callback,void *);
extern void		X11_MenuItemCheck(X11_widget, boolean);
extern X11_widget	X11_SubMenuItem(X11_widget,char *,char *,char);
#define X11_MenuItemSetString(i,t)	X11_LabelSetString(i,t)
#define X11_MenuItemSetCall(i,c,d)	X11_ButtonSetCall(i,c,d)

extern void		X11_MenuItemRadio(X11_widget,char **,char *,int,
						X11_int_call,void *);
extern void		X11_MenuItemRadioCheck(X11_widget);


#endif /*_widget_menuitem_h_included*/
