/* $Id: colorsP.h,v 1.2 1997/06/10 15:29:23 archer Exp $
 *	COLORSP.H - Color management within the widgets
 *
 * Colors are specified by ressources. Most of the time, that is...
 */
#ifndef _widgets_colorsp_h_included
#define _widgets_colorsp_h_included
#include <Widget/Color.h>


/**
 ** Prototypes
 **/
extern X11_color	x11__color_resource(char *,X11_color);
extern void		x11__color_init(void);


#endif /*_widgets_colorsp_h_included*/
