/* $Id: List.h,v 1.4 1997/05/11 13:01:30 archer Exp $
 *	LIST.H - Widget scrollable List
 *
 * List have many purposes...
 */
#ifndef _widget_list_h_included
#define _widget_list_h_included
#include <Widget/Widgets.h>
#include <Widget/Icon.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_List(X11_widget,char *,int,int);
extern void		X11_ListRefill(X11_widget);
extern void		X11_ListSetLine(X11_widget,int,char *,void *);
extern void		X11_ListSetTabs(X11_widget,char *);
extern int		X11_ListSize(X11_widget);
extern void		X11_ListTruncate(X11_widget,int);


#endif /*_widget_list_h_included*/
