/* $Id: Shell.h,v 1.1 1997/04/15 10:04:16 archer Exp $
 *	SHELL.H - Basic Shell widgets and functions
 */
#ifndef _widget_shell_h_included
#define _widget_shell_h_included
#include <Widget/Widgets.h>
#include <Widget/Icon.h>


/**
 ** Useful Constants
 **/
#define X11_SHELL_MODELESS_DIALOG	((X11_widget)-1)


/**
 ** Prototypes about the Shell widget itself
 **/
extern X11_widget	X11_Shell(char *,char *,X11_widget);
extern void		X11_ShellHints(X11_widget,int,int,unsigned int,unsigned int);
extern void		X11_ShellIcon(X11_widget, X11_icon *);
extern void		X11_ShellIsDialog(X11_widget);
extern void		X11_ShellPosition(X11_widget,int *,int *,unsigned int *,unsigned int *);
extern void		X11_ShellSetExit(X11_widget,X11_callback,void *);
extern void		X11_ShellTitle(X11_widget,char *);


/**
 ** Prototypes about the Shell managing a widget
 **/
extern void		X11_ShellFocus(X11_widget);
extern void		X11_ShellGeneralIcon(X11_icon *);
extern void		X11_ShellShuffle(X11_widget);
extern X11_widget	X11_ShellUnder(X11_widget);

/**
 ** Accelerators management
 **/
extern void	X11_Accelerator(X11_widget,KeySym,int,X11_callback,void *);
extern void	X11_AcceleratorCancel(X11_widget);


#endif /*_widget_shell_h_included*/
