/* $Id: SeparatorP.h,v 1.1 1997/04/15 10:07:14 archer Exp $
 *	SEPARATORP.H - Private structures needed by separators
 */
#ifndef _widget_separatorp_h_included
#define _widget_separatorp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/Separator.h>


/**
 ** Private types
 **/
typedef struct {
	x11__core	core;
} x11__separator;
#define W_SEPARATOR	((x11__separator *)wdg)


#endif /*_widget_separatorp_h_included*/
