/* $Id: ButtonP.h,v 1.1 1997/04/15 10:07:05 archer Exp $
 *	BUTTONP.H - Structure used for command buttons
 *
 * The Command button always has a callback and so on.
 */
#ifndef _widget_buttonp_h_included
#define _widget_buttonp_h_included
#include <Widget/LabelP.h>
#include <Widget/Button.h>


/**
 ** Private types
 **/
typedef struct {
	x11__label	label;
/*
 * Private parts
 */
	X11_callback	action;
	void		*userdata;
} x11__button;
#define W_BUTTON	((x11__button *)wdg)


/**
 ** Prototypes
 **	(functions used by sub-classes of Button)
 **/
extern void	x11__button_geometry(x11__button *);
extern void	x11__button_expose(X11_widget, XEvent *);


#endif /*_widget_buttonp_h_included*/
