/* $Id: UnWM.h,v 1.1 1997/04/15 10:04:20 archer Exp $
 *	UNWM.H - Widgets that are not managed by WM
 *
 */
#ifndef _widget_unwm_h_included
#define _widget_unwm_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern void	X11_WidgetRootLocation(X11_widget,int *,int *);


#endif /*_widget_unwm_h_included*/
