/* $Id: ArrowP.h,v 1.1 1997/04/17 13:20:43 archer Exp $
 *	ARROWP.H - Drawing of arrows
 *
 * These arrows are used in Scrollbars
 */
#ifndef _widget_arrowp_h_included
#define _widget_arrowp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/Arrow.h>
#include <Widget/Signal.h>


/**
 ** Arrow type
 **/
typedef struct {
	x11__core	core;
/*
 * Specials
 */
	X11_timer	timer;
	X11_callback	callback;
	void		*user_data;
	char		direction;
} x11__arrow;
#define W_ARROW	((x11__arrow *)wdg)


/**
 ** Internal prototypes
 **/
extern void	x11__draw_arrow(int,GC,GC,GC);


#endif /*_widget_arrowp_h_included*/
