/* $Id: MenuItemP.h,v 1.2 1997/05/01 15:22:13 archer Exp $
 *	MENUITEMP.H - Structure used for menu items
 *
 * MenuItems are subclassed from MenuButtons, with the optional part that
 * they might have accelerators and shortcuts attached to them.
 */
#ifndef _widget_menuitemp_h_included
#define _widget_menuitemp_h_included
#include <Widget/MenuButtonP.h>
#include <Widget/MenuItem.h>


/**
 ** Private types
 **/
typedef struct {
	x11__menubutton	menub;
/*
 * Inside the menu?
 */
	X11_int_call	callback;
	int		underline_start,
			underline_ends;
	int		checkbox_size;
	char		key;
	boolean		checkbox;
	boolean		checked;
} x11__menuitem;
#define W_MENUITEM	((x11__menuitem *)wdg)
#define C_MENUITEM	((x11__menuitem *)child)


/**
 ** Private variables
 **/
extern char	x11__menuitem_class[];


/**
 ** Private prototype
 **/
extern long	x11__menu_index(X11_widget wdg);


#endif /*_widget_menuitemp_h_included*/
