/* $Id: strdup.c,v 1.1 1997/04/15 10:09:54 archer Exp $
 *	STRDUP.C - Subroutines not included with your OS
 */
#include "config.h"


#ifndef HAVE_STRDUP
/**
 ** STRDUP
 **	Return a malloc-ated copy of the string
 **/
char *strdup(const char *s)
{
char	*m;

	if ((m = malloc(strlen(s)+1)) != 0)
		strcpy(m, s);
	return m;
}
#endif
