/* $Id: IconP.h,v 1.1 1997/04/15 10:07:08 archer Exp $
 *	ICONP.H - Icons structures and prototypes
 *
 * Support for mono-chrome and color pixmaps
 */
#ifndef _widget_iconp_h_included
#define _widget_iconp_h_included
#include <Widget/Icon.h>


/**
 ** Private prototypes
 **/
extern Pixmap	x11__color_pixmap(X11_icon *);
extern Pixmap	x11__data_bitmap(X11_icon *);
extern void	x11__icon_draw(Window,X11_icon *,
					unsigned long,unsigned long, int,int);
extern void	x11__icon_init(void);
extern void	x11__icon_sized(X11_icon *);


#endif /*_widget_icon_h_included*/
