/* $Id: misc.h,v 1.1 1997/06/10 16:08:25 archer Exp $
 *	MISC.H - Various prototypes
 */
#include <stdio.h>


/**
 ** Global variables
 **/
extern char	work[8192];


/**
 ** Prototypes
 **/
extern int	file_gets(FILE *);
