/* $Id: Scrollbar.h,v 1.2 1997/04/22 13:22:16 archer Exp $
 *	SCROLLBAR.H - Widget Scrollbar
 */
#ifndef _widget_scrollbar_h_included
#define _widget_scrollbar_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_ScrollbarNew(X11_widget,char *,boolean);
extern long		X11_ScrollbarValue(X11_widget);


/**
 ** Macros for prototypes
 **/
#define		X11_ScrollbarHorizontal(a) X11_ScrollbarNew(a,"hbar",True);
#define		X11_ScrollbarVertical(a)   X11_ScrollbarNew(a,"vbar",False);


#endif /*_widget_scrollbar_h_included*/
