/* $Id: Template.c,v 1.1 1997/04/15 10:09:50 archer Exp $
 *	XXXX.C - Handle XXXX widgets
 *
 */
#include <Widget/XXXXP.h>


/**
 ** X11_XXXX
 **	Create a XXXX object
 **/
X11_widget X11_XXXX(X11_widget parent, char *name)
{
x11__XXXX	*object;
/*
 * Get resource
 */
	object = allocator(x11__XXXX);
	x11__core_setup(&object->core, parent, "XXXX", name);
/*
 * Initialize fields
 */
	object->core.event_mask = ExposureMask;
/*
 * Fetch X11 resources
 */
	x11__resource_basic(&object->core);
/*
 * Ok, create X11 window
 */
	return (X11_widget)object;
}
