/* $Id: Perror.h,v 1.1 1997/04/15 10:04:14 archer Exp $
 *	PERROR.H - Fatal system error.
 *
 * If we are crashing!
 */
#ifndef _widget_perror_h_included
#define _widget_perror_h_included


/**
 ** Prototype of perror
 **/
extern void	X11_Perror(char *);


#endif /*_widget_perror_h_included*/
