/* $Id: LabelP.h,v 1.1 1997/04/15 10:07:09 archer Exp $
 *	LABELP.H - Structure used for labelling
 *
 * Labels can have a lot of appearances
 */
#ifndef _widget_labelp_h_included
#define _widget_labelp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/fontsP.h>
#include <Widget/Label.h>


/**
 ** Private type
 **/
typedef struct {
	x11__core	core;
/*
 * Private parts
 */
	XFontStruct	*font;
	X11_icon	*icon;
	char		*string;
	x11__size	w, h,
			w_str, h_str;
	boolean		icon_above;
	boolean		multi_line;
} x11__label;
#define W_LABEL		((x11__label *)wdg)
#define C_LABEL		((x11__label *)child)


/**
 ** Prototypes
 **/
extern void	x11__label_cleanup(X11_widget,XEvent *);
extern void	x11__label_draw(x11__label *,unsigned long,unsigned long);
extern void	x11__label_draw_active(x11__label *);
extern void	x11__label_fit(x11__label *);
extern void	x11__label_resources(x11__label *,char *,char *);
extern void	x11__label_size(x11__label *);


#endif /*_widget_labelp_h_included*/
