/* $Id: LorMap.h,v 1.2 1997/06/27 12:41:51 archer Exp $
 *	LORMAP.H - Lorenai Map widget
 */
#ifndef _lorenai_map_h_
#define _lorenai_map_h_
#include <Widget/Widgets.h>


/*
 * Just a prototype
 */
extern X11_widget	Lor_map(X11_widget);
extern void		Lor_Map_Refresh(X11_widget);


#endif /*_lorenai_map_h_*/
