/* $Id: Scroller.h,v 1.1 1997/04/17 13:20:28 archer Exp $
 *	SCROLLER.H - Widget Scroller for windows
 *
 * Scrolling logical viewports
 */
#ifndef _widget_scroller_h_included
#define _widget_scroller_h_included
#include <Widget/Widgets.h>
#include <Widget/Scrollbar.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Scroller(X11_widget,boolean,boolean);


#endif /*_widget_scroller_h_included*/
