/* $Id: BoundingBox.h,v 1.1 1997/04/15 10:04:04 archer Exp $
 *	BoundingBox.H - Bounding box container
 *
 * May have or not have a label
 */
#ifndef _widget_boundingbox_h_included
#define _widget_boundingbox_h_included
#include <Widget/Label.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_BoundingBox(X11_widget,char *,char *);


#endif /*_widget_boundingbox_h_included*/
