/* $Id: Widgets.h,v 1.2 1997/04/17 13:20:29 archer Exp $
 *	WIDGETS.H - Global Widget Library definitions
 *
 * Defines the basic X11_widget type and the global widget functions.
 */
#ifndef _widget_widgets_h_included
#define _widget_widgets_h_included
/* this is needed by GCC, it seems */
#define __USE_FIXED_PROTOTYPES__
#ifndef NeedFunctionPrototypes
#define NeedFunctionPrototypes 1
#endif
#ifndef NeedNestedPrototypes
#define NeedNestedPrototypes 1
#endif
#include <X11/Xlib.h>


#ifndef True
#define True (-1)
#endif
#ifndef False
#define False 0
#endif
/**
 ** Types
 **/
typedef char		boolean;
typedef struct x11__o	x11__core,
			*X11_widget;
typedef void		(*X11_callback)(X11_widget,void *);
typedef void		(*X11_string_call)(X11_widget,void *,char *);
typedef void		(*X11_int_call)(X11_widget,void *,long);


/**
 ** Client-side Prototypes
 **/
extern boolean	X11_Initialize(char *,char *,int *,char **);
extern void	X11_Destroy(X11_widget);


#endif /*_widget_widgets_h_included*/
