/* $Id: miscP.h,v 1.1 1997/04/17 13:20:46 archer Exp $
 *	MISCP.H - scale
 *
 * All that appears in two or more widgets
 */
#ifndef _widgets_miscp_h_included
#define _widgets_miscp_h_included


/**
 ** Prototypes
 **/
extern long	x11__rescale_long(long,long,long);


#endif /*_widgets_miscp_h_included*/
