/* $Id: ChoiceMenuP.h,v 1.4 1997/05/01 15:22:12 archer Exp $
 *	CHOICEMENUP.H - Choice button, with a menu to select all that
 */
#ifndef _widget_choicemenup_h_included
#define _widget_choicemenup_h_included
#include <Widget/ChoiceMenu.h>
#include <Widget/MenuButtonP.h>


/**
 ** Special type
 **/
typedef struct {
	x11__menubutton	menub;
/*
 * Associated Widget
 */
	X11_widget	menu;
/*
 * Callbacks for ChoiceMenu
 */
	X11_int_call	callback;
} x11__choicemenu;
#define W_CHOICEMENU	((x11__choicemenu *)wdg)


#endif /*_widget_choicemenup_h_included*/
