/* $Id: Malloc.c,v 1.1 1997/04/15 10:09:39 archer Exp $
 *	MALLOC.C - Memory allocation function
 */
#include <Widget/Perror.h>
#include "config.h"
#include <stdlib.h>
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif


/**
 ** X11_WIDGET_MALLOC
 **	Malloc, check, and bzero
 **/
char *x11__widget_malloc(int size)
{
char	*m;
/*
 * Mallocator should be XMalloc?
 */
	if ((m = malloc(size)) == 0)
		X11_Perror("Allocating memory");
	memset(m, 0, size);
	return m;
}
