/* $Id: SignalP.h,v 1.1 1997/04/15 10:07:15 archer Exp $
 *	SIGNALP.H - Signal handling internal types
 *
 * Two signal hanlders are installed. One for SIGCHLD, one for SIGALRM. The
 * latter is optional, because SIGALRMs while debugging are a nuisance.
 */
#ifndef _widget_signalp_h_included
#define _widget_signalp_h_included
#include <Widget/Signal.h>


/**
 ** Private types
 **/
typedef struct x11__widget_child {
	struct x11__widget_child	*next;
	pid_t				pid;
	X11_exit			on_exit;
	void				*user_data;
} x11__sigchld;


/**
 ** Prototypes
 **/
extern void	x11__signal_init(boolean);


#endif /*_widget_signalp_h_included*/
