/* $Id: BoundingBoxP.h,v 1.1 1997/04/15 10:07:04 archer Exp $
 *	BOUNDINGBOXP.H - Structure used for bounding box
 *
 * The Bounding box is a sub-class of Label. It draws a box, with a title,
 * surrounding another object
 */
#ifndef _widget_boundingboxp_h_included
#define _widget_boundingboxp_h_included
#include <Widget/LabelP.h>
#include <Widget/BoundingBox.h>


/**
 ** Private types
 **/
typedef struct {
	x11__label	label;
} x11__boundingbox;
#define W_BOUNDINGBOX	((x11__boundingbox *)wdg)


#endif /*_widget_boundingboxp_h_included*/
