/* $Id: MenuBarP.h,v 1.1 1997/04/15 10:07:10 archer Exp $
 *	MENUBARP.H - Private structures needed by Menu Bars
 *
 * In fact, a menubar is a very simple object... Its behaviour is similar to
 * a menu, except that its always mapped and laid out horizontally...
 */
#ifndef _widget_menubarp_h_included
#define _widget_menubarp_h_included
#include <Widget/MenuP.h>
#include <Widget/MenuBar.h>


/**
 ** Private types
 **/
typedef struct {
	x11__menu	menu;
} x11__menubar;
#define W_MENUBAR	((x11__menubar *)wdg)


/**
 ** Private variables
 **/
extern char	x11__menubar_class[];


#endif /*_widget_menubarp_h_included*/
