/* $Id: PopupP.h,v 1.1 1997/04/15 10:07:13 archer Exp $
 *	POPUP.H - Shell without WM-interaction
 *
 * The Popup should be able to work alone, outside of the WM. In fact, a popup
 * uses OverrideRedirect and XGrabPointer, to avoid any nasty interaction.
 */
#ifndef _widget_popupp_h_included
#define _widget_popupp_h_included
#include <Widget/ShellP.h>
#include <Widget/Popup.h>
#include <Widget/fontsP.h>


/**
 ** Private types
 **/
typedef struct {
	x11__shell	shell;
/*
 * Popup needs to be able to redraw the title.
 */
	XFontStruct	*font;
	unsigned int	w, h;
	int		length;
} x11__popup;
#define W_POPUP		((x11__popup *)wdg)


#endif /*_widget_popupp_h_included*/
