/* $Id: TextView.h,v 1.2 1997/04/24 12:59:19 archer Exp $
 *	TEXTVIEW.H - Text Viewer Widget
 *
 * A quick and dirty hack for debugging purposes
 */
#ifndef _widget_textview_h_included
#define _widget_textview_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_TextView(X11_widget, char *, char **);
extern void		X11_TextViewRefresh(X11_widget, int);


#endif /*_widget_textview_h_included*/
