/* $Id: UnWMP.h,v 1.1 1997/04/15 10:07:17 archer Exp $
 *	UNWM.H - Windows without WM-interaction
 *
 */
#ifndef _widget_unwmp_h_included
#define _widget_unwmp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/UnWM.h>


/**
 ** Prototypes
 **/
extern void	x11__nowm_create(X11_widget,XEvent *);


#endif /*_widget_unwmp_h_included*/
