/* $Id: LorMapP.h,v 1.1 1997/06/10 16:08:20 archer Exp $
 *	LORMAPP.H - Lorenai Map widget internals
 */
#ifndef _lorenai_mapp_h_
#define _lorenai_mapp_h_
#include "LorMap.h"
#include <Widget/WidgetsP.h>
#include <Widget/ScrollerP.h>


/*
 * Lorenai Map structure
 */
typedef struct {
	x11__core	core;
/*
 * Related elements
 */
	x11__scroller	*scroller;
	int		w, h;
	int		delta_x, delta_y;
	int		scale;
} x11__lor_map;
#define W_LORMAP	((x11__lor_map *)wdg)


#endif /*_lorenai_mapp_h_*/
