/* $Id: TextViewP.h,v 1.1 1997/04/23 13:41:06 archer Exp $
 *	TEXTVIEWP.H - Private structures needed by TextView
 */
#ifndef _widget_textviewp_h_included
#define _widget_textviewp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/TextView.h>


/**
 ** Private types
 **/
typedef struct {
	x11__core	core;
/*
 * Private part
 */
	char		**lines;
	XFontStruct	*font;
	int		h;
} x11__textview;
#define W_TEXTVIEW	((x11__textview *)wdg)


#endif /*_widget_textviewp_h_included*/
