/* $Id: Separator.h,v 1.1 1997/04/15 10:04:15 archer Exp $
 *	SEPARATOR.H - Horizontal line separator
 */
#ifndef _widget_separator_h_included
#define _widget_separator_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Separator(X11_widget);


#endif /*_widget_separator_h_included*/
