/* $Id: TextField.h,v 1.1 1997/04/15 10:04:18 archer Exp $
 *	TEXTFIELD.H - Editable text field widget
 */
#ifndef _widget_textfield_h_included
#define _widget_textfield_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_TextField(X11_widget,char *,int,int);
extern void		X11_TextFieldCallback(X11_widget,X11_string_call,void *);
extern void		X11_TextFieldSelected(X11_widget);
extern void		X11_TextFieldSet(X11_widget,char *);
extern char		*X11_TextFieldValue(X11_widget);


#endif /*_widget_textfield_h_included*/
