/* $Id: MenuBar.h,v 1.1 1997/04/15 10:04:12 archer Exp $
 *	MENUBAR.H - Widget Menu bar (hold button menus, plus optional help menu)
 */
#ifndef _widget_menubar_h_included
#define _widget_menubar_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_MenuBar(X11_widget);


#endif /*_widget_menubar_h_included*/
