/* $Id: FormP.h,v 1.1 1997/04/15 10:07:08 archer Exp $
 *	FORMP.H - Private structures needed by the various Forms managers
 *
 * Elementary managers
 */
#ifndef _widget_formp_h_included
#define _widget_formp_h_included
#include <Widget/WidgetsP.h>
#include <Widget/Form.h>


/**
 ** Private types
 **/
typedef struct {
	x11__core	core;
} x11__form;


#endif /*_widget_formp_h_included*/
