/* $Id: strerror.c,v 1.1 1997/04/15 10:09:55 archer Exp $
 *	STRERROR.C - Subroutines not included with your OS
 */
#include "config.h"


#ifndef HAVE_STRERROR
/**
 ** STRERROR
 **	Return the error text from errno
 **/
char *strerror(int no)
{
extern char	*sys_errlist[];
extern int	sys_nerr;

	if (no < 0 || no >= sys_nerr)
		return "Unknown error";
	return sys_errlist[no];
}
#endif /*!HAVE_STRERROR*/
