/* $Id: clipP.h,v 1.1 1997/04/15 10:07:18 archer Exp $
 *	CLIPP.H - Control selection (clipboard) mechanism
 *
 * Selection occur mostly between text areas.
 */
#ifndef _widget_clipp_h_included
#define _widget_clipp_h_included
#include <Widget/WidgetsP.h>


/**
 ** Prototypes
 **/
extern void	x11__clipboard_clear(XEvent *);
extern void	x11__clipboard_owner(X11_widget,x11__handler);
extern void	x11__clipboard_request(XSelectionRequestEvent *);
extern char	*x11__clipboard_get(void);
extern void	x11__clipboard_string(char *,int);


#endif /*_widget_clipp_h_included*/
