/* $Id: Manage.h,v 1.1 1997/04/15 10:04:10 archer Exp $
 *	MANAGE.H - Widget common management functions
 *
 */
#ifndef _widget_manage_h_included
#define _widget_manage_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern boolean	X11_IsManaged(X11_widget);
extern void	X11_Manage(X11_widget);
extern void	X11_UnManage(X11_widget);


#endif /*_widget_manage_h_included*/
