/* $Id: Popup.h,v 1.1 1997/04/15 10:04:15 archer Exp $
 *	POPUP.H - Popup widgets and functions
 *
 * Popups are shells without a WM management.
 */
#ifndef _widget_popup_h_included
#define _widget_popup_h_included
#include <Widget/Widgets.h>
#include <Widget/Shell.h>


/**
 ** Prototypes
 **/
extern X11_widget		X11_Popup(char *,char *);
extern void			X11_PopupDisplay(X11_widget);


#endif /*_widget_popup_h_included*/
