/* $Id: Color.h,v 1.1 1997/06/10 15:28:32 archer Exp $
 *	COLORS.H - Color management for those widgets that use it in parameters
 *
 * Colors are specified by ressources. Most of the time, that is...
 */
#ifndef _widgets_colors_h_included
#define _widgets_colors_h_included


/**
 ** Special type
 **/
typedef unsigned long	X11_color;


/**
 ** Prototypes
 **/
extern X11_color	X11_ColorByName(char *, X11_color);


#endif /*_widgets_colors_h_included*/
