/* $Id: CheckBox.h,v 1.1 1997/04/15 10:04:06 archer Exp $
 *	CHECKBOX.H - Widget CheckBox / RadioBox
 */
#ifndef _widget_checkbox_h_included
#define _widget_checkbox_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_CheckBox(X11_widget,char *,char *,boolean,
							X11_callback,void *);
extern void		X11_CheckBoxSet(X11_widget,boolean);
extern boolean		X11_CheckBoxValue(X11_widget);


#endif /*_widget_checkbox_h_included*/
