/* $Id: Button.h,v 1.2 1997/06/27 12:40:56 archer Exp $
 *	BUTTON.H - Widget Push Button
 *
 * A button, you push, and you get a function executed.
 */
#ifndef _widget_button_h_included
#define _widget_button_h_included
#include <Widget/Widgets.h>
#include <Widget/Label.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_Button(X11_widget,char *,char *,X11_icon *,
							X11_callback,void *);
extern void		X11_ButtonDefaults(X11_widget);
extern void		X11_ButtonSetCall(X11_widget,X11_callback,void *);


#endif /*_widget_button_h_included*/
