/* $Id: panP.h,v 1.1 1997/04/22 13:22:39 archer Exp $
 *	PANP.H - Prototypes for panning function
 *
 * These functions are used by everyone
 */
#ifndef _widget_panp_h_included
#define _widget_panp_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern void	x11__pan_window(X11_widget,int,int);


#endif /*_widget_panp_h_included*/
