/* $Id: ChoiceMenu.h,v 1.3 1997/05/01 15:21:55 archer Exp $
 *	CHOICEMENU.H - Choice button, with a menu to select all that
 */
#ifndef _widget_choicemenu_h_included
#define _widget_choicemenu_h_included
#include <Widget/Widgets.h>


/**
 ** Prototypes
 **/
extern X11_widget	X11_ChoiceMenu(X11_widget,char *,char **,int,
						X11_int_call,void *);
extern void		X11_ChoiceMenuNew(X11_widget,char **,int);


#endif /*_widget_choicemenu_h_included*/
