/* $Id: Events.h,v 1.1 1997/04/15 10:04:07 archer Exp $
 *	EVENTS.H - Process Events and handlers
 *
 * Defines the stream callback & main event functions
 * widget functions.
 */
#ifndef _widget_events_h_included
#define _widget_events_h_included
#include <Widget/Widgets.h>


/**
 ** Types
 **/
typedef void	(*X11_input)(int,void *);


/**
 ** Prototypes
 **/
extern void	X11_MainLoop(void);
extern void	X11_InputDescriptor(int,X11_input,void *);
extern void	X11_OutputDescriptor(int,X11_input,void *);
extern void	X11_ClosedDescriptor(int);


#endif /*_widget_events_h_included*/
